#!/usr/bin/python
# Plots sensitivity output from Adjoint

import sys
import numpy as np
import matplotlib.pyplot as plt
import argparse

class DataSet(object):

    def __init__(self,fname=None,var=0):
        if fname is not None:
            self.fname = fname
            self.readFile(var)
        else:
            self.fname = ""

    def readFile(self,var=0):

        # Open and read file
        f = open(self.fname,'r')
        lines = f.readlines()

        # Initial places to store data. Work out domain size later...
        xp = []
        yp = []
        ss = []

        for l in lines:
            ll = l.strip().split()
            xp.append(int(ll[0]))
            yp.append(int(ll[1]))
            ss.append(float(ll[2+var]))

        self.xn = max(xp)
        self.yn = max(yp)

        self.data = np.zeros((self.yn,self.xn))

        for i in range(len(xp)):
            self.data[yp[i]-1,xp[i]-1] = ss[i]

    def __add__(self,other):
        if self.xn!=other.xn or self.yn!=other.yn:
            raise TypeError("Unsupported addition on DataSet type")
        out = DataSet()
        out.data = self.data + other.data
        out.xn = self.xn
        out.yn = other.yn
        out.fname = self.fname+" + "+other.fname
        return out

    def __sub__(self,other):
        if self.xn!=other.xn or self.yn!=other.yn:
            raise TypeError("Unsupported subtraction on DataSet type")
        out = DataSet()
        out.data = self.data - other.data
        out.xn = self.xn
        out.yn = other.yn
        out.fname = self.fname+" - "+other.fname
        return out

if __name__=="__main__":
    # Define parser and arguments
    parser = argparse.ArgumentParser(description='Plot sensitivity maps from adjoint and forward models')
    parser.add_argument('fname',type=str,metavar='FILENAME')
    parser.add_argument('-c',type=str,metavar='FILENAME')
    parser.add_argument('-l',type=float,metavar='LOWER')
    parser.add_argument('-u',type=float,metavar='UPPER')
    parser.add_argument('-p',dest='var',action='store_const',const=1,default=0)
    args = parser.parse_args()

    # Load data and difference if necessary
    data1 = DataSet(args.fname,args.var)
    if args.c is not None:
        data2 = DataSet(args.c,args.var)
        data = data1 - data2
    else:
        data = data1

    ncontours = 30

    if args.u is not None and args.l is not None:
        plt.contourf(data.data, args.l+(args.u-args.l)/ncontours*np.arange(ncontours))
    else:
        plt.contourf(data.data,ncontours)
    plt.colorbar()
    plt.show()
